defmodule DecentralisedBookIndex.Metadata.AuthorRole do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer

  alias DecentralisedBookIndex.Metadata

  postgres do
    table "author_role"
    repo DecentralisedBookIndex.Repo
  end

  actions do
    defaults [:read, :update, :destroy]

    create :create do
      primary? true
      accept [:order, :role]
      argument :author_id, :uuid

      change fn changeset, _ ->
        author_id = changeset.arguments.author_id

        if author_id != nil do
          {:ok, author} = DecentralisedBookIndex.Metadata.get_author_by_id(author_id)

          Ash.Changeset.force_change_attribute(changeset, :author_id, author.id)
        else
          changeset
        end
      end
    end

    read :by_id do
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end
  end

  attributes do
    uuid_primary_key :id

    attribute :role, :string do
      allow_nil? true
      public? true
    end

    attribute :order, :integer do
      allow_nil? false
      public? true
    end

    timestamps()
  end

  relationships do
    belongs_to :book, Metadata.Book do
      allow_nil? true
    end

    belongs_to :author, Metadata.Author do
      allow_nil? true
    end
  end
end
